package com.dreamcodex.ti.component;

import com.dreamcodex.ti.iface.MapChangeListener;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * Created with IntelliJ IDEA.
 * User: Rasmus
 * Date: 06-05-13
 * Time: 20:43
 */
public class AnalyzeCharTransDialog extends JDialog implements ActionListener, MapChangeListener, WindowListener, ListSelectionListener {

    private final MapEditor mapEditor;
    private final HashMap<String, Image> charImages;
    ArrayList<TransChar> sortedTransCharList;
    private boolean vertical = true;
    private JRadioButton verticalButton;
    private JRadioButton horizontalButton;
    private JTable jTable;
    private CharTransTableModel tableModel;
    private JButton refreshButton;
    private JButton closeButton;

    private class TransChar {

        private int fromChar;
        private int toChar;
        private int count;

        public TransChar(int fromChar, int toChar) {
            this.fromChar = fromChar;
            this.toChar = toChar;
            count = 1;
        }

        public int getFromChar() {
            return fromChar;
        }

        public int getToChar() {
            return toChar;
        }

        public int getCount() {
            return count;
        }

        public void incCount() {
            count++;
        }
    }

    private class TransCharComparator implements Comparator<TransChar> {
        public int compare(TransChar tc1, TransChar tc2) {
            return tc2.getCount() - tc1.getCount();
        }
    }

    private class CharTransTableModel extends AbstractTableModel {

        Map<String, TransChar> transCharMap;

        CharTransTableModel() {
            buildMap();
        }

        public void refresh() {
            buildMap();
            fireTableDataChanged();
            mapEditor.removeAllHighlights();
            mapEditor.redrawCanvas();
        }

        public void buildMap() {
            transCharMap = new HashMap<String, TransChar>();
            int[][] mapData = mapEditor.getMapData(mapEditor.getCurrentMapId());
            if (mapData.length > 1 && mapData[0].length > 1) {
                for (int y = (vertical ? 1 : 0); y < mapData.length; y++) {
                    for (int x = (vertical ? 0 : 1); x < mapData[0].length; x++) {
                        int fromChar = mapData[y][x];
                        int toChar = vertical ? mapData[y - 1][x] : mapData[y][x - 1];
                        String key = Integer.toString(fromChar) + "-" + Integer.toString(toChar);
                        TransChar transChar = transCharMap.get(key);
                        if (transChar != null) {
                            transChar.incCount();
                        }
                        else {
                            transCharMap.put(key, new TransChar(fromChar, toChar));
                        }
                    }
                }
            }
            sortedTransCharList = new ArrayList<TransChar>(transCharMap.values());
            Collections.sort(sortedTransCharList, new TransCharComparator());
        }

        public int getRowCount() {
            return sortedTransCharList.size();
        }

        public int getColumnCount() {
            return 4;
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            if (columnIndex == 0) {
                return rowIndex;
            }
            if (columnIndex == 1) {
                return sortedTransCharList.get(rowIndex);
            }
            if (columnIndex == 2) {
                return sortedTransCharList.get(rowIndex);
            }
            if (columnIndex == 3) {
                return sortedTransCharList.get(rowIndex).getCount();
            }
            return null;
        }

        public String getColumnName(int column) {
            if (column == 0) {
                return "#";
            }
            if (column == 1) {
                return "From";
            }
            if (column == 2) {
                return "To";
            }
            if (column == 3) {
                return "Count";
            }
            return null;
        }

        public Class<?> getColumnClass(int columnIndex) {
            if (columnIndex == 0) {
                return Integer.class;
            }
            if (columnIndex == 1) {
                return TransChar.class;
            }
            if (columnIndex == 2) {
                return TransChar.class;
            }
            if (columnIndex == 3) {
                return Integer.class;
            }
            return null;
        }
    }

    private class CenteredRenderer extends DefaultTableCellRenderer {
        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
            setHorizontalAlignment(SwingConstants.CENTER);
            return super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
        }
    }

    private class TransCharRenderer extends DefaultTableCellRenderer {

        boolean from;

        public TransCharRenderer(boolean from) {
            this.from = from;
        }

        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
            super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
            TransChar transChar = (TransChar) value;
            if (from) {
                setText(Integer.toString(transChar.getFromChar()));
                setIcon(new ImageIcon(charImages.get(getCharKey(transChar.getFromChar()))));
            }
            else {
                setText(Integer.toString(transChar.getToChar()));
                setIcon(new ImageIcon(charImages.get(getCharKey(transChar.getToChar()))));
            }
            return this;
        }
    }

    protected String getCharKey(int charnum) {
        return ("ch" + charnum);
    }

    public AnalyzeCharTransDialog(JFrame parent, MapEditor mapEditor, HashMap<String, Image> charImages) {
        super(parent, "Analyze Character Transitions");
        this.mapEditor = mapEditor;
        this.charImages = charImages;
        setLayout(new BorderLayout());
        verticalButton = new JRadioButton("Top to Bottom", true);
        horizontalButton = new JRadioButton("Right to Left");
        ButtonGroup radioButtonGroup = new ButtonGroup();
        radioButtonGroup.add(verticalButton);
        radioButtonGroup.add(horizontalButton);
        JPanel radioButtonPanel = new JPanel();
        radioButtonPanel.add(verticalButton);
        radioButtonPanel.add(horizontalButton);
        add(radioButtonPanel, BorderLayout.NORTH);
        jTable = new JTable();
        tableModel = new CharTransTableModel();
        jTable.setModel(tableModel);
        jTable.getColumnModel().getColumn(0).setCellRenderer(new CenteredRenderer());
        jTable.getColumnModel().getColumn(1).setCellRenderer(new TransCharRenderer(true));
        jTable.getColumnModel().getColumn(2).setCellRenderer(new TransCharRenderer(false));
        jTable.setRowHeight(22);
        JScrollPane jScrollPane = new JScrollPane(jTable);
        add(jScrollPane, BorderLayout.CENTER);
        refreshButton = new JButton("Refresh");
        closeButton = new JButton("Close");
        JPanel buttonPanel = new JPanel();
        buttonPanel.add(refreshButton);
        buttonPanel.add(closeButton);
        add(buttonPanel, BorderLayout.SOUTH);
        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        pack();
        setLocationRelativeTo(this);
        setSize(300, 500);
        setVisible(true);
        addWindowListener(this);
        verticalButton.addActionListener(this);
        horizontalButton.addActionListener(this);
        refreshButton.addActionListener(this);
        closeButton.addActionListener(this);
        jTable.setRowSelectionAllowed(true);
        jTable.setColumnSelectionAllowed(false);
        jTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        jTable.getSelectionModel().addListSelectionListener(this);
        mapEditor.addMapChangeListener(this);
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == verticalButton) {
            vertical = true;
            tableModel.refresh();
        }
        else if (e.getSource() == horizontalButton) {
            vertical = false;
            tableModel.refresh();
        }
        else if (e.getSource() == refreshButton) {
            tableModel.refresh();
        }
        else if (e.getSource() == closeButton) {
            this.dispose();
        }
    }

    public void valueChanged(ListSelectionEvent e) {
        if (sortedTransCharList != null && jTable.getSelectedRow() != -1 && sortedTransCharList.size() > jTable.getSelectedRow()) {
            mapEditor.removeAllHighlights();
            TransChar transChar =  sortedTransCharList.get(jTable.getSelectedRow());
            int[][] mapData = mapEditor.getMapData(mapEditor.getCurrentMapId());
            if (mapData.length > 1 && mapData[0].length > 1) {
                for (int y = (vertical ? 1 : 0); y < mapData.length; y++) {
                    for (int x = (vertical ? 0 : 1); x < mapData[0].length; x++) {
                        int fromChar = mapData[y][x];
                        int toChar = vertical ? mapData[y - 1][x] : mapData[y][x - 1];
                        if (fromChar == transChar.getFromChar() && toChar == transChar.getToChar()) {
                            mapEditor.highlightCell(x, y);
                            mapEditor.highlightCell(vertical ? x : x - 1, vertical ? y - 1 : y);
                        }
                    }
                }
            }
            mapEditor.redrawCanvas();
        }
    }

    public void mapChanged() {
        tableModel.refresh();
    }

    public void windowOpened(WindowEvent e) {
    }

    public void windowClosing(WindowEvent e) {
        mapEditor.removeAllHighlights();
        mapEditor.removeMapChangeListener(this);
    }

    public void windowClosed(WindowEvent e) {
    }

    public void windowIconified(WindowEvent e) {
    }

    public void windowDeiconified(WindowEvent e) {
    }

    public void windowActivated(WindowEvent e) {
    }

    public void windowDeactivated(WindowEvent e) {
    }
}
